#vim:set et sts=4 sw=4:
# -*- encoding: utf-8 -*-
#
# Fonts Tweak Tool
#
# Copyright (c) 2011-2012 Jian Ni <jni@redhat.com>
# Copyright (c) 2012 Red Hat, Inc.
#
#This program is free software: you can redistribute it and/or 
#modify it under the terms of the GNU Lesser General Public 
#License as published by the Free Software Foundation, either
#version 3 of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import string
import gi
import gettext
import locale
from collections import OrderedDict
from gi.repository import Gtk
from gi.repository import GObject
from gi.repository import Easyfc
from xml.sax.saxutils import quoteattr
from xml.sax.saxutils import escape

__all__ = (
	    "FontsTweakTool",
          )

def N_(s):
    return s

alias_names = ['sans-serif', 'serif', 'monospace', 'cursive', 'fantasy']
sample_text = N_('The quick brown fox jumps over the lazy dog')
GETTEXT_PACKAGE = 'fonts-tweak-tool'
LOCALEDIR = '/usr/share/locale'

class LangList:

    def __init__(self, parent):
        self.langlist = OrderedDict()
        self.parent_window = parent
        path = os.path.dirname(os.path.realpath(__file__))
        localefile = os.path.join(path, '..', 'data', 'locale-list')

        # Add "any" language to the list.
        self.langlist[''] = N_('Default')
        try:
            fd = open(localefile, "r")
        except:
            try:
                # XXX: need to polish it with the better way.
		fd = open("/usr/share/fonts-tweak-tool/locale-list", "r")
            except:
                raise RuntimeError, ("Cannot find locale-list")

        while True:
            line = fd.readline()
            if not line:
                break
	    tokens = string.split(line)
            lang = str(tokens[0]).split('.')[0].replace('_', '-')
            self.langlist[lang] = string.join(tokens[3:], ' ')

    def show_dialog(self):
        builder = Gtk.Builder()
	builder.set_translation_domain(GETTEXT_PACKAGE)
        path = os.path.dirname(os.path.realpath(__file__))
        uifile = os.path.join(path, '..', 'data', 'fontstools.ui')
        if not os.path.isfile(uifile):
            # need to polish it with the better way
            uifile = "/usr/share/fonts-tweak-tool/fontstools.ui"
        builder.add_from_file(uifile)
        self.dialog = builder.get_object("dialog2")
        self.dialog.set_transient_for(self.parent_window)
        self.langStore = builder.get_object("lang_and_locale_list")

        for l in self.langlist.keys():
            iter = self.langStore.append()
            self.langStore.set_value(iter, 0, l)
            self.langStore.set_value(iter, 1, self.langlist[l])

        self.langView = builder.get_object("lang_view")
        col = Gtk.TreeViewColumn(None, Gtk.CellRendererText(), text=1)
        self.langView.append_column(col)
        self.dialog.show_all()
        return self.dialog.run()

    def close_dialog(self):
        self.dialog.destroy()
        self.dialog = None

    def get_selection(self):
        lang = None
        fullName = None
        if self.dialog != None:
            model, iterator = self.langView.get_selection().get_selected()
            if iterator:
                lang = model.get_value(iterator, 0)
                fullName = self.langStore.get_value(iterator, 1)
	        return (lang, fullName)
            else:
                return None

class FontsTweakTool:

    def selectionChanged(self, *args):
	selection = self.lang_view.get_selection()
        model, iter = selection.get_selected()
        if iter == None:
            self.note_book.set_current_page(1)
            self.removelang_button.set_sensitive(False)
        else:
            lang = model.get_value(iter, 1)
            for n in alias_names:
                self.render_combobox(lang, n)
	    self.note_book.set_current_page(0)
            self.removelang_button.set_sensitive(True)
            self.font_changed = False
	
    def add_language(self, desc, lang):
        retval = True
        model = self.lang_view.get_model()
        iter = model.get_iter_first()
        while iter != None:
            n, l = model.get(iter, 0, 1)
            if l == lang:
                retval = False
                break
            iter = model.iter_next(iter)
        if retval == True:
            iter = self.lang_list.append()
            self.lang_list.set_value(iter, 0, desc)
            self.lang_list.set_value(iter, 1, lang)
        else:
            iter = None

        return iter

    def addlangClicked(self, *args):
        response = self.languages.show_dialog()
	self.language_selected = True
        if response != Gtk.ResponseType.CANCEL:
            selection = self.languages.get_selection()
	    if selection != None:
	        lang, desc = selection
		iter = self.add_language(desc, lang)
            	if iter == None:
                    print "%s has already been added.\n" % lang
            	else:
                    model = self.lang_view.get_model()
                    path = model.get_path(iter)
                    self.lang_view.set_cursor(path, None, False)

        self.languages.close_dialog()

    def removelangClicked(self, *args):
        selection = self.lang_view.get_selection()
        model, iter = selection.get_selected()
        if iter != None:
            lang = model.get(iter, 1)[0]
            self.lang_list.remove(iter)
            self.note_book.set_current_page(1)
            self.removelang_button.set_sensitive(False)
            self.config.remove_aliases(lang)

    def fontChanged(self, combobox, *args):
        if self.__initialized == False:
            return
	selection = self.lang_view.get_selection()
        model, iter = selection.get_selected()
        if iter != None:
            lang = model.get(iter, 1)[0]
            model = combobox.get_model()
            iter = combobox.get_active_iter()
            if iter != None:
                font = model.get(iter, 0)[0]
                iter = model.get_iter_first()
                alias = model.get(iter, 0)[0]
                self.config.remove_alias(lang, alias)
                a = Easyfc.Alias.new(alias)
                try:
                    a.set_font(font)
                    self.config.add_alias(lang, a)
                except gi._glib.GError:
                    pass
                self.render_label(combobox, lang)
                self.font_changed = True

    def closeClicked(self, *args):
	if self.language_selected and not self.font_changed:
	    dialog = Gtk.MessageDialog(self.window, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                        Gtk.MessageType.WARNING, Gtk.ButtonsType.OK,
                        "The added language will be discard after closing")
            dialog.show_all()
            response = dialog.run()
            dialog.destroy()
	
	if self.font_changed:
	    dialog = Gtk.MessageDialog(self.window, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
           		Gtk.MessageType.WARNING, Gtk.ButtonsType.YES_NO,
                	"Do you want to save your changes before closing?")
	    dialog.show_all()
	    response = dialog.run()
	    
	    if response == Gtk.ResponseType.YES:
	      	self.applyClicked()	
	    dialog.destroy()
	Gtk.main_quit()

    def applyClicked(self, *args):
        try:
            self.config.save()
        except gi._glib.GError, e:
            if e.domain != 'ezfc-error-quark' and e.code != 6:
                raise
            else:
                print "%s: %s" % (sys.argv[0], e)
        Gtk.main_quit()

    def translate_text(self, text, lang):
        if self.translations.has_key(lang) == False:
            self.translations[lang] = gettext.translation(
                domain=GETTEXT_PACKAGE,
                localedir=LOCALEDIR,
                languages=[lang.replace('-', '_')],
                fallback=True,
                codeset="utf8")
        return unicode(self.translations[lang].gettext(text), "utf8")

    def render_label(self, combobox, lang):
        model = combobox.get_model()
        iter = combobox.get_active_iter()
        if iter != None:
            font = model.get(iter, 0)[0]
            # Work around for PyGObject versions 3.0.3 and 3.1.0,
            # which decode strings in Gtk.TreeModel when retrieval.
            # This behavior was reverted in 3.1.1:
            # http://git.gnome.org/browse/pygobject/commit/?id=0285e107
            if type(font) is not unicode:
                font = unicode(font, "utf8")
            iter = model.get_iter_first()
            alias = model.get(iter, 0)[0]
            self.label[alias].set_markup(
                "<span font_family=%s font_size=\"small\">%s</span>" % (
                    quoteattr(font),
                    escape(self.translate_text(sample_text, lang))))

    def render_combobox(self, lang, alias):
        if self.fontslist.has_key(lang) == False:
            self.fontslist[lang] = {}
        if self.fontslist[lang].has_key(alias) == False:
            self.fontslist[lang][alias] = Easyfc.get_fonts_list(lang, alias)
        self.lists[alias].clear()
        self.lists[alias].append([alias])
        for f in self.fontslist[lang][alias]:
	    self.lists[alias].append([f])
        fn = None
        for a in self.config.get_aliases(lang):
            if a.get_name() == alias:
                fn = a.get_font()
                break
        if fn != None:
            model = self.combobox[alias].get_model()
            iter = model.get_iter_first()
            while iter != None:
                f = model.get(iter, 0)[0]
		if type(fn) is not unicode:
		    fontname = unicode(fn, "utf8") 
                if f == fontname:
                    self.combobox[alias].set_active_iter(iter)
                    break
                iter = model.iter_next(iter)
        else:
            self.combobox[alias].set_active(0)
        self.render_label(self.combobox[alias], lang)

    def __init__(self):
        self.__initialized = False
	self.font_changed = False
	self.language_selected = False
        builder = Gtk.Builder()
	builder.set_translation_domain(GETTEXT_PACKAGE)
        path = os.path.dirname(os.path.realpath(__file__))
        uifile = os.path.join(path, '..', 'data', 'fontstools.ui')
        if not os.path.isfile(uifile):
            # need to polish it with the better way
            uifile = "/usr/share/fonts-tweak-tool/fontstools.ui"
	builder.add_from_file(uifile) 
        self.window = builder.get_object("dialog1")
        self.window.connect("destroy", Gtk.main_quit)
	self.window.set_title("fonts-tweak-tool")
        self.window.set_size_request(640, 480)        
	
	self.scrollwindow = builder.get_object("scrolledwindow1")
	self.scrollwindow.set_min_content_width(200)
	self.lang_view = builder.get_object("treeview1")
        self.lang_list = builder.get_object("lang_list")
        column = Gtk.TreeViewColumn(None, Gtk.CellRendererText(), text=0)
	self.lang_view.append_column(column)

	self.note_book = builder.get_object("notebook1")
	self.note_book.set_current_page(1)

        self.fontslist = {}

        self.combobox = {}
	self.combobox['sans-serif'] = builder.get_object("sans_combobox")
	self.combobox['serif'] = builder.get_object("serif_combobox")
	self.combobox['monospace'] = builder.get_object("monospace_combobox")
	self.combobox['cursive'] = builder.get_object("cursive_combobox")
	self.combobox['fantasy'] = builder.get_object("fantasy_combobox")

        self.label = {}
	self.label['sans-serif'] = builder.get_object("sans_label")
	self.label['serif'] = builder.get_object("serif_label")
	self.label['monospace'] = builder.get_object("monospace_label")
	self.label['cursive'] = builder.get_object("cursive_label")
	self.label['fantasy'] = builder.get_object("fantasy_label")

        for f in alias_names:
            renderer_text = Gtk.CellRendererText()
            self.combobox[f].pack_start(renderer_text, True)
            self.combobox[f].add_attribute(renderer_text, "text", 0)
            self.combobox[f].connect('changed', self.fontChanged)

        self.lists = {}
        self.lists['sans-serif'] = builder.get_object("sans_fonts_list")
	self.lists['serif'] = builder.get_object("serif_fonts_list")
	self.lists['monospace'] = builder.get_object("monospace_fonts_list")
	self.lists['cursive'] = builder.get_object("cursive_fonts_list")
	self.lists['fantasy'] = builder.get_object("fantasy_fonts_list")

	self.close_button = builder.get_object("button2")
	self.close_button.connect("clicked", self.closeClicked)
  
	self.addlang_button = builder.get_object("add-lang")
        self.addlang_button.connect("clicked", self.addlangClicked)

        self.removelang_button = builder.get_object("remove-lang")
        self.removelang_button.connect("clicked", self.removelangClicked)
        self.removelang_button.set_sensitive(False)
	
	self.apply_button = builder.get_object("button1")
	self.apply_button.connect("clicked", self.applyClicked)

     	selection = self.lang_view.get_selection()
        selection.connect("changed", self.selectionChanged)

        self.languages = LangList(self.window)
        self.data = {}
        self.translations = {}

	Easyfc.init()

        self.config = Easyfc.Config()
        self.config.set_name("fontstweak")
        try:
            self.config.load()
        except gi._glib.GError, e:
            if e.domain != 'ezfc-error-quark' or e.code != 7:
                raise

        for l in self.config.get_language_list():
            # XXX: need to take care of the KeyError exception?
            desc = self.languages.langlist[l]
            self.add_language(desc, l)
            for a in self.config.get_aliases(l):
                an = a.get_name()
                self.render_combobox(l, an)
	
	self.__initialized = True

def main(argv):
    try:
        locale.setlocale(locale.LC_ALL, '')
    except Locale.Error, e:
        os.environ['LC_ALL'] = 'C'
        locale.setlocale(locale.LC_ALL, '')

    gettext.bind_textdomain_codeset(GETTEXT_PACKAGE, locale.nl_langinfo(locale.CODESET))
    gettext.bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR)
    gettext.textdomain(GETTEXT_PACKAGE)
    tool = FontsTweakTool()
    tool.window.show_all()
    tool.font_changed = False
    Gtk.main()
